@@ -1,7 +1,7 @@
 /*
  * Copyright 2005-2011 Gentoo Foundation
  * Distributed under the terms of the GNU General Public License v2
- * $Header: /var/cvsroot/gentoo-projects/portage-utils/libq/scandirat.c,v 1.4 2012/01/16 01:10:32 vapier Exp $
+ * $Header: /var/cvsroot/gentoo-projects/portage-utils/libq/scandirat.c,v 1.6 2013/09/29 10:25:25 vapier Exp $
  *
  * Copyright 2005-2010 Ned Ludd        - <solar@gentoo.org>
  * Copyright 2005-2011 Mike Frysinger  - <vapier@gentoo.org>
@@ -15,6 +15,12 @@
 
 #if !defined(HAVE_SCANDIRAT)
 
+#if defined(_DIRENT_HAVE_D_RECLEN)
+# define reclen(de) ((de)->d_reclen)
+#else
+# define reclen(de) (sizeof(*(de)))
+#endif
+
 static int scandirat(int dir_fd, const char *dir, struct dirent ***dirlist,
 	int (*filter)(const struct dirent *),
 	int (*compar)(const struct dirent **, const struct dirent **))
@@ -39,7 +45,7 @@ static int scandirat(int dir_fd, const char *dir, struct dirent ***dirlist,
 			continue;
 
 		ret = realloc(ret, sizeof(*ret) * (cnt + 1));
-		ret[cnt++] = xmemdup(de, de->d_reclen);
+		ret[cnt++] = xmemdup(de, reclen(de));
 	}
 	*dirlist = ret;
 
