@@ -1,7 +1,7 @@
 /*
  * Copyright 2005-2010 Gentoo Foundation
  * Distributed under the terms of the GNU General Public License v2
- * $Header: /var/cvsroot/gentoo-projects/portage-utils/qcache.c,v 1.41 2012/10/28 04:16:19 vapier Exp $
+ * $Header: /var/cvsroot/gentoo-projects/portage-utils/qcache.c,v 1.46 2013/09/29 18:40:22 vapier Exp $
  *
  * Copyright 2006 Thomas A. Cort - <tcort@gentoo.org>
  */
@@ -47,7 +47,7 @@ static const char * const qcache_opts_help[] = {
 	COMMON_OPTS_HELP
 };
 
-static const char qcache_rcsid[] = "$Id: qcache.c,v 1.41 2012/10/28 04:16:19 vapier Exp $";
+static const char qcache_rcsid[] = "$Id: qcache.c,v 1.46 2013/09/29 18:40:22 vapier Exp $";
 #define qcache_usage(ret) usage(ret, QCACHE_FLAGS, qcache_long_opts, qcache_opts_help, lookup_applet_idx("qcache"))
 
 /********************************************************************/
@@ -342,6 +342,7 @@ portage_cache *qcache_read_cache_file(const char *filename)
 		goto err;
 	}
 
+	buf = NULL;
 	len = sizeof(*ret) + s.st_size + 1;
 	ret = xzalloc(len);
 
@@ -529,7 +530,8 @@ int qcache_traverse(void (*func)(qcache_data*))
 		xasprintf(&pkgpath, "%s/%s", portdir, categories[i]->d_name);
 
 		if (-1 == (num_pkg = scandir(pkgpath, &packages, qcache_file_select, alphasort))) {
-			warnp("Found a cache dir, but unable to process %s", pkgpath);
+			if (errno != ENOENT)
+				warnp("Found a cache dir, but unable to process %s", pkgpath);
 			free(categories[i]);
 			free(pkgpath);
 			continue;
@@ -596,7 +598,8 @@ int qcache_traverse(void (*func)(qcache_data*))
 
 					qcache_free_data(data.cache_data);
 				} else
-					warn("unable to read cache '%s'\n\tperhaps you need to `emerge --metadata` or `emerge --regen` ?", cachepath);
+					warnp("unable to read cache '%s'\n"
+					      "\tperhaps you need to `egencache -j 4` ?", cachepath);
 
 				free(ebuilds[k]);
 				free(cachepath);
@@ -786,7 +789,6 @@ void qcache_stats(qcache_data *data)
 	memset(keywords, 0, archlist_count * sizeof(*keywords));
 	if (read_keywords(data->cache_data->KEYWORDS, keywords) < 0) {
 		warn("Failed to read keywords for %s%s/%s%s%s", BOLD, data->category, BLUE, data->ebuild, NORM);
-		free(keywords);
 		return;
 	}
 
