@@ -1,15 +1,15 @@
 /*
- * Copyright 2005-2010 Gentoo Foundation
+ * Copyright 2005-2013 Gentoo Foundation
  * Distributed under the terms of the GNU General Public License v2
- * $Header: /var/cvsroot/gentoo-projects/portage-utils/qdepends.c,v 1.57 2012/11/10 06:35:21 vapier Exp $
+ * $Header: /var/cvsroot/gentoo-projects/portage-utils/qdepends.c,v 1.66 2013/09/29 22:19:39 vapier Exp $
  *
  * Copyright 2005-2010 Ned Ludd        - <solar@gentoo.org>
- * Copyright 2005-2010 Mike Frysinger  - <vapier@gentoo.org>
+ * Copyright 2005-2013 Mike Frysinger  - <vapier@gentoo.org>
  */
 
 #ifdef APPLET_qdepends
 
-#define QDEPENDS_FLAGS "drpaNk:Q:" COMMON_FLAGS
+#define QDEPENDS_FLAGS "drpafNk:Q:" COMMON_FLAGS
 static struct option const qdepends_long_opts[] = {
 	{"depend",    no_argument, NULL, 'd'},
 	{"rdepend",   no_argument, NULL, 'r'},
@@ -18,6 +18,7 @@ static struct option const qdepends_long_opts[] = {
 	{"query",      a_argument, NULL, 'Q'},
 	{"name-only", no_argument, NULL, 'N'},
 	{"all",       no_argument, NULL, 'a'},
+	{"format",    no_argument, NULL, 'f'},
 	COMMON_LONG_OPTS
 };
 static const char * const qdepends_opts_help[] = {
@@ -28,9 +29,10 @@ static const char * const qdepends_opts_help[] = {
 	"Query reverse deps",
 	"Only show package name",
 	"Show all DEPEND info",
+	"Pretty format specified depend strings",
 	COMMON_OPTS_HELP
 };
-static const char qdepends_rcsid[] = "$Id: qdepends.c,v 1.57 2012/11/10 06:35:21 vapier Exp $";
+static const char qdepends_rcsid[] = "$Id: qdepends.c,v 1.66 2013/09/29 22:19:39 vapier Exp $";
 #define qdepends_usage(ret) usage(ret, QDEPENDS_FLAGS, qdepends_long_opts, qdepends_opts_help, lookup_applet_idx("qdepends"))
 
 static char qdep_name_only = 0;
@@ -43,9 +45,7 @@ typedef enum {
 	DEP_OR = 3,
 	DEP_GROUP = 4
 } dep_type;
-#ifdef EBUG
 static const char * const _dep_names[] = { "NULL", "NORM", "USE", "OR", "GROUP" };
-#endif
 
 struct _dep_node {
 	dep_type type;
@@ -58,23 +58,27 @@ struct _dep_node {
 };
 typedef struct _dep_node dep_node;
 
+static const dep_node null_node = {
+	.type = DEP_NULL,
+};
+
 /* prototypes */
-#define dep_dump_tree(r) _dep_dump_tree(r,0)
-void _dep_dump_tree(dep_node *root, int space);
-dep_node *dep_grow_tree(char *depend);
+#ifdef NDEBUG
+# define dep_dump_tree(r)
+#else
+# define dep_dump_tree(r) _dep_print_tree(stdout, r, 0)
+#endif
+_q_static void _dep_print_tree(FILE *fp, const dep_node *root, size_t space);
 void dep_burn_tree(dep_node *root);
-void dep_prune_use(dep_node *root, char *use);
-char *dep_flatten_tree(dep_node *root);
-dep_node *_dep_grow_node(dep_type type, char *info, size_t info_len);
-void _dep_attach(dep_node *root, dep_node *attach_me, int type);
-void _dep_flatten_tree(dep_node *root, char *buf, size_t *pos);
-void _dep_burn_node(dep_node *node);
+char *dep_flatten_tree(const dep_node *root);
+_q_static void _dep_attach(dep_node *root, dep_node *attach_me, int type);
+_q_static void _dep_flatten_tree(const dep_node *root, char *buf, size_t *pos);
+_q_static void _dep_burn_node(dep_node *node);
 int qdepends_main_vdb(const char *depend_file, int argc, char **argv);
 int qdepends_vdb_deep(const char *depend_file, const char *query);
 
 #ifdef EBUG
-void print_word(char *ptr, int num);
-void print_word(char *ptr, int num)
+_q_static void print_word(const char *ptr, size_t num)
 {
 	while (num--)
 		printf("%c", *ptr++);
@@ -82,7 +86,8 @@ void print_word(char *ptr, int num)
 }
 #endif
 
-dep_node *_dep_grow_node(dep_type type, char *info, size_t info_len)
+_q_static dep_node *
+_dep_grow_node(dep_type type, const char *info, size_t info_len)
 {
 	dep_node *ret;
 	size_t len;
@@ -96,8 +101,7 @@ dep_node *_dep_grow_node(dep_type type, char *info, size_t info_len)
 			info_len = strlen(info);
 		len += info_len + 1;
 	}
-	ret = xmalloc(len);
-	memset(ret, 0x00, len);
+	ret = xzalloc(len);
 
 	ret->type = type;
 	if (info) {
@@ -140,10 +144,11 @@ void _dep_attach(dep_node *root, dep_node *attach_me, int type)
 	}
 }
 
-dep_node *dep_grow_tree(char *depend)
+_q_static dep_node *dep_grow_tree(const char *depend)
 {
+	bool saw_whitespace;
 	signed long paren_balanced;
-	char *word, *ptr, *p;
+	const char *ptr, *word;
 	int curr_attach;
 	dep_node *ret, *curr_node, *new_node;
 	dep_type prev_type;
@@ -154,9 +159,6 @@ dep_node *dep_grow_tree(char *depend)
 	curr_attach = _DEP_NEIGH;
 	word = NULL;
 
-	p = strrchr(depend, '\n');
-	if (p != NULL) *p = 0;
-
 #define _maybe_consume_word(t) \
 	do { \
 	if (!word) break; \
@@ -174,8 +176,10 @@ dep_node *dep_grow_tree(char *depend)
 	word = NULL; \
 	} while (0)
 
+	saw_whitespace = true;
 	for (ptr = depend; *ptr; ++ptr) {
 		if (isspace(*ptr)) {
+			saw_whitespace = true;
 			_maybe_consume_word(DEP_NORM);
 			continue;
 		}
@@ -191,6 +195,8 @@ dep_node *dep_grow_tree(char *depend)
 			continue;
 		}
 		case '|': {
+			if (!saw_whitespace)
+				break;
 			if (ptr[1] != '|') {
 				warnf("Found a | but not ||");
 				goto error_out;
@@ -202,6 +208,8 @@ dep_node *dep_grow_tree(char *depend)
 		}
 		case '(': {
 			++paren_balanced;
+			if (!saw_whitespace)
+				break;
 			if (prev_type == DEP_OR || prev_type == DEP_USE) {
 				_maybe_consume_word(DEP_NORM);
 				prev_type = DEP_NULL;
@@ -218,6 +226,8 @@ dep_node *dep_grow_tree(char *depend)
 		}
 		case ')': {
 			--paren_balanced;
+			if (!saw_whitespace)
+				break;
 			_maybe_consume_word(DEP_NORM);
 
 			if (curr_node->parent == NULL) {
@@ -232,6 +242,7 @@ dep_node *dep_grow_tree(char *depend)
 			if (!word)
 				word = ptr;
 		}
+		saw_whitespace = false;
 
 		/* fall through to the paren failure below */
 		if (paren_balanced < 0)
@@ -249,7 +260,7 @@ dep_node *dep_grow_tree(char *depend)
 
 #undef _maybe_consume_word
 
-	return ret;
+	return ret ? : xmemdup(&null_node, sizeof(null_node));
 
 error_out:
 	warnf("DEPEND: %s", depend);
@@ -260,26 +271,62 @@ error_out:
 	return NULL;
 }
 
-#ifdef EBUG
-void _dep_dump_tree(dep_node *root, int space)
+_q_static void _dep_print_tree(FILE *fp, const dep_node *root, size_t space)
 {
-	int spaceit = space;
+	size_t s;
+
 	assert(root);
-	if (root->type == DEP_NULL) goto this_node_sucks;
+	if (root->type == DEP_NULL)
+		goto this_node_sucks;
 
-	while (spaceit--) printf("\t");
-	printf("Node [%s]: ", _dep_names[root->type]);
+	for (s = space; s; --s)
+		fprintf(fp, "\t");
+
+	if (verbose > 1)
+		fprintf(fp, "Node [%s]: ", _dep_names[root->type]);
 	/*printf("Node %p [%s] %p %p %p: ", root, _dep_names[root->type], root->parent, root->neighbor, root->children);*/
-	if (root->info) printf("'%s'", root->info);
-	printf("\n");
+	if (root->type == DEP_OR)
+		fprintf(fp, "|| (");
+	if (root->info) {
+		fprintf(fp, "%s", root->info);
+		/* If there is only one child, be nice to one-line: foo? ( pkg ) */
+		if (root->type == DEP_USE)
+			fprintf(fp, "? (");
+	}
+	fprintf(fp, "\n");
 
-	if (root->children) _dep_dump_tree(root->children, space+1);
-this_node_sucks:
-	if (root->neighbor) _dep_dump_tree(root->neighbor, space);
+	if (root->children)
+		_dep_print_tree(fp, root->children, space+1);
+
+	if (root->type == DEP_OR || root->type == DEP_USE) {
+		for (s = space; s; --s)
+			fprintf(fp, "\t");
+		fprintf(fp, ")\n");
+	}
+ this_node_sucks:
+	if (root->neighbor)
+		_dep_print_tree(fp, root->neighbor, space);
+}
+
+_q_static bool dep_print_depend(FILE *fp, const char *depend)
+{
+	dep_node *dep_tree;
+
+	dep_tree = dep_grow_tree(depend);
+	if (dep_tree == NULL)
+		return false;
+
+	if (!quiet)
+		fprintf(fp, "DEPEND=\"\n");
+
+	_dep_print_tree(fp, dep_tree, 1);
+
+	dep_burn_tree(dep_tree);
+	if (!quiet)
+		fprintf(fp, "\"\n");
+
+	return true;
 }
-#else
-void _dep_dump_tree(_q_unused_ dep_node *root, _q_unused_ int space) {;}
-#endif
 
 void dep_burn_tree(dep_node *root)
 {
@@ -289,7 +336,7 @@ void dep_burn_tree(dep_node *root)
 	_dep_burn_node(root);
 }
 
-void dep_prune_use(dep_node *root, char *use)
+_q_static void dep_prune_use(dep_node *root, const char *use)
 {
 	if (root->neighbor) dep_prune_use(root->neighbor, use);
 	if (root->type == DEP_USE) {
@@ -306,7 +353,7 @@ void dep_prune_use(dep_node *root, char *use)
 	if (root->children) dep_prune_use(root->children, use);
 }
 
-void _dep_flatten_tree(dep_node *root, char *buf, size_t *pos)
+void _dep_flatten_tree(const dep_node *root, char *buf, size_t *pos)
 {
 	if (root->type == DEP_NULL) goto this_node_sucks;
 	if (root->type == DEP_NORM) {
@@ -328,7 +375,7 @@ this_node_sucks:
 	if (root->neighbor) _dep_flatten_tree(root->neighbor, buf, pos);
 }
 
-char *dep_flatten_tree(dep_node *root)
+char *dep_flatten_tree(const dep_node *root)
 {
 	static char flat[8192];
 	size_t pos = 0;
@@ -418,7 +465,7 @@ _q_static int qdepends_main_vdb_cb(q_vdb_pkg_ctx *pkg_ctx, void *priv)
 
 	dep_burn_tree(dep_tree);
 
-	return EXIT_SUCCESS;
+	return 1;
 }
 
 _q_static int qdepends_vdb_deep_cb(q_vdb_pkg_ctx *pkg_ctx, void *priv)
@@ -432,7 +479,7 @@ _q_static int qdepends_vdb_deep_cb(q_vdb_pkg_ctx *pkg_ctx, void *priv)
 	char depend[16384], use[8192];
 	dep_node *dep_tree;
 
-	IF_DEBUG(warn("matched %s/%s", catname, pkgname));
+	IF_DEBUG(warn("matched %s/%s for %s", catname, pkgname, state->depend_file));
 
 	if (!eat_file_at(pkg_ctx->fd, state->depend_file, depend, sizeof(depend)))
 		return 0;
@@ -476,7 +523,7 @@ _q_static int qdepends_vdb_deep_cb(q_vdb_pkg_ctx *pkg_ctx, void *priv)
 	}
 	dep_burn_tree(dep_tree);
 
-	return EXIT_SUCCESS;
+	return 1;
 }
 
 int qdepends_main(int argc, char **argv)
@@ -486,7 +533,8 @@ int qdepends_main(int argc, char **argv)
 		.argv = argv,
 	};
 	q_vdb_pkg_cb *cb;
-	int i;
+	int i, ret;
+	bool do_format = false;
 	const char *query = NULL;
 	const char *depend_file;
 	const char *depend_files[] = { "DEPEND", "RDEPEND", "PDEPEND", NULL, NULL };
@@ -507,26 +555,39 @@ int qdepends_main(int argc, char **argv)
 		case 'a': depend_file = NULL; break;
 		case 'Q': query = optarg; break;
 		case 'N': qdep_name_only = 1; break;
+		case 'f': do_format = true; break;
 		}
 	}
-	if ((argc == optind) && (query == NULL))
+	if ((argc == optind) && (query == NULL) && !do_format)
 		qdepends_usage(EXIT_FAILURE);
 
+	if (do_format) {
+		while (optind < argc) {
+			if (!dep_print_depend(stdout, argv[optind++]))
+				return EXIT_FAILURE;
+			if (optind < argc)
+				fprintf(stdout, "\n");
+		}
+		return EXIT_SUCCESS;
+	}
+
 	state.depend_file = depend_file;
 	state.query = query;
 	cb = query ? qdepends_vdb_deep_cb : qdepends_main_vdb_cb;
 
 	if (!depend_file) {
-		int ret = 0;
+		ret = 0;
 		for (i = 0; depend_files[i]; ++i) {
 			printf(" %s*%s %s\n", GREEN, NORM, depend_files[i]);
 			state.depend_file = depend_files[i];
-			ret += q_vdb_foreach_pkg(cb, &state, NULL);
+			ret |= q_vdb_foreach_pkg(cb, &state, NULL);
 		}
-		return ret;
-	}
+	} else
+		ret = q_vdb_foreach_pkg(cb, &state, NULL);
 
-	return q_vdb_foreach_pkg(cb, &state, NULL);
+	if (!ret && !quiet)
+		warn("no matches found for your query");
+	return ret ? EXIT_SUCCESS : EXIT_FAILURE;
 }
 
 #else
